
From: Richard Purdie <rpurdie@rpsys.net>

To solve touchscreen interference problems devices like the Sharp Zaurus
SL-C3000 need to know the length of the horitzontal sync pulses.  This patch
adds a hook to pxafb so the touchscreen driver can function correctly.

Signed-Off-By: Richard Purdie <rpurdie@rpsys.net>
Signed-off-by: Antonino Daplas <adaplas@pol.net>
Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 drivers/video/pxafb.c            |   32 ++++++++++++++++++++++++++++++++
 drivers/video/pxafb.h            |    2 ++
 include/asm-arm/arch-pxa/pxafb.h |    1 +
 3 files changed, 35 insertions(+)

diff -puN drivers/video/pxafb.c~pxafb-add-hsync-time-reporting-hook drivers/video/pxafb.c
--- 25/drivers/video/pxafb.c~pxafb-add-hsync-time-reporting-hook	Thu Sep  1 17:04:54 2005
+++ 25-akpm/drivers/video/pxafb.c	Thu Sep  1 17:04:54 2005
@@ -468,6 +468,36 @@ static inline unsigned int get_pcd(unsig
 }
 
 /*
+ * Some touchscreens need hsync information from the video driver to
+ * function correctly. We export it here.
+ */
+static inline void set_hsync_time(struct pxafb_info *fbi, unsigned int pcd)
+{
+	unsigned long long htime;
+
+	if ((pcd == 0) || (fbi->fb.var.hsync_len == 0)) {
+		fbi->hsync_time=0;
+		return;
+	}
+
+	htime = (unsigned long long)get_lcdclk_frequency_10khz() * 10000;
+	do_div(htime, pcd * fbi->fb.var.hsync_len);
+	fbi->hsync_time = htime;
+}
+
+unsigned long pxafb_get_hsync_time(struct device *dev)
+{
+	struct pxafb_info *fbi = dev_get_drvdata(dev);
+
+	/* If display is blanked/suspended, hsync isn't active */
+	if (!fbi || (fbi->state != C_ENABLE))
+		return 0;
+
+	return fbi->hsync_time;
+}
+EXPORT_SYMBOL(pxafb_get_hsync_time);
+
+/*
  * pxafb_activate_var():
  *	Configures LCD Controller based on entries in var parameter.  Settings are
  *	only written to the controller if changes were made.
@@ -631,6 +661,7 @@ static int pxafb_activate_var(struct fb_
 	fbi->reg_lccr1 = new_regs.lccr1;
 	fbi->reg_lccr2 = new_regs.lccr2;
 	fbi->reg_lccr3 = new_regs.lccr3;
+	set_hsync_time(fbi, pcd);
 	local_irq_restore(flags);
 
 	/*
@@ -907,6 +938,7 @@ pxafb_freq_transition(struct notifier_bl
 
 	case CPUFREQ_POSTCHANGE:
 		pcd = get_pcd(fbi->fb.var.pixclock);
+		set_hsync_time(fbi, pcd);
 		fbi->reg_lccr3 = (fbi->reg_lccr3 & ~0xff) | LCCR3_PixClkDiv(pcd);
 		set_ctrlr_state(fbi, C_ENABLE_CLKCHANGE);
 		break;
diff -puN drivers/video/pxafb.h~pxafb-add-hsync-time-reporting-hook drivers/video/pxafb.h
--- 25/drivers/video/pxafb.h~pxafb-add-hsync-time-reporting-hook	Thu Sep  1 17:04:54 2005
+++ 25-akpm/drivers/video/pxafb.h	Thu Sep  1 17:04:54 2005
@@ -83,6 +83,8 @@ struct pxafb_info {
 	u_int			reg_lccr2;
 	u_int			reg_lccr3;
 
+	unsigned long	hsync_time;
+
 	volatile u_char		state;
 	volatile u_char		task_state;
 	struct semaphore	ctrlr_sem;
diff -puN include/asm-arm/arch-pxa/pxafb.h~pxafb-add-hsync-time-reporting-hook include/asm-arm/arch-pxa/pxafb.h
--- 25/include/asm-arm/arch-pxa/pxafb.h~pxafb-add-hsync-time-reporting-hook	Thu Sep  1 17:04:54 2005
+++ 25-akpm/include/asm-arm/arch-pxa/pxafb.h	Thu Sep  1 17:04:54 2005
@@ -66,3 +66,4 @@ struct pxafb_mach_info {
 
 };
 void set_pxa_fb_info(struct pxafb_mach_info *hard_pxa_fb_info);
+unsigned long pxafb_get_hsync_time(struct device *dev);
_
